<?php
namespace Teknisa\OPL\Modules\AutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\OPL\Util\Attachment;
use Teknisa\OPL\Util\OplUtilities;

class Sup05100AnaliseAFControllerReport extends \Zeedhi\Framework\Controller\Report 
{
    const EMAIL_ENVIADO = true;
    const EMAIL_NAO_ENVIADO = false;
    protected $reportService;
    protected $qr2Strategy;
    protected $birtReportsPath;
    private $message;
    private $emailEnviado;
    private $relatorioGerado;
    private $oplUtilities;
    private $urlRelatorio;

    public function __construct($reportService, $qr2Strategy, OplUtilities $oplUtilities, $environment) 
    {
        $this->reportService        = $reportService;
        $this->qr2Strategy          = $qr2Strategy;
        $this->isEmailEnviado       = false;
        $this->isRelatorioGerado    = false;
        $this->oplUtilities         = $oplUtilities;
        $this->environment          = $environment;
    }
    
    protected $parameterMappingBirt = array(
        'CDFILIAL'       => 'P_CDFILIAL',
        'NRAUTOFORN'     => 'P_NRAUTOFORN',
        'CDFILLANC'      => 'P_CDFILLANC',
        'IMPRIMEOBS'     => 'P_IDEXIBEOBSFIL',
        'IMPRIMEOBSPROD' => 'P_IMPRIMEOBSPROD',
        'CDFORNECED'     => 'P_CDFORNECED',
        'NRSEQAPURAF'    => 'P_NRSEQAPURAF',
        "NRORG"          => 'NRORG'
    );
    
    protected $parameterMappingQr2 = array(
        'TIPO'              => 'P_TIPO',
        'CDFORNECED'        => 'P_CDFORNECED',
        'CDFILLANC'         => 'P_CDFILLANC',
        'L_NRSEQAPURAF'     => 'L_NRSEQAPURAF',
        'NRSEQENVIOAFORN'   => 'P_NRSEQENVIOAFORN',
        "L_CDFILDEST"       => 'L_CDFILDEST',
        'L_NRAUTOFORN'      => 'L_NRAUTOFORN'
    );
    
    protected $reportMapping = array();
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);
        $parameter = $row['STRATEGY'] == 'QR2' ? $this->parameterMappingQr2 : $this->parameterMappingBirt;
        
        $this->reportMapping[$this->stNomeRel] = array (
            'strategy' => $row['STRATEGY'],
            'parameterMapping' => $parameter
        );
    }
    
    public function geraRelatorio($row) {
        $localResponse = new DTO\Response();
        $statusEnvio = false;
        
        $row["CDFILIAL"] = $this->environment->getCdFilial();
        
        $this->setRelatorioGerado(false);
        parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $localResponse);
        
        return $this->isRelatorioGerado() ? $this->getUrlRelatorio() : null;
    }
    
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $urlRelatorio) {
        $this->setRelatorioGerado(true);
        $this->setUrlRelatorio($urlRelatorio);
    }
    
    public function enviaEmail($row) {
        $isStringAttachment = true;
        
        switch($row["TYPE"]){
            case "PDF":
                $tipo = 'application/pdf';
                break;
            case "XLS":
                $tipo = 'application/xls';
                break;
            case "RTF":
                $tipo = 'application/rtf';
                break;
            case "TXT":
                $tipo = 'application/txt';
                break;
            default:
                $tipo = 'application/txt';
        }
        
		try {
            $enderecos = explode(",", $row['DSEMAILCOT']);
            $enderecosCopia = explode(",", $row['DSEMAILCOTCOP']);
            $assunto = $row["ASSUNTO"];
            $corpoEmail = $this->oplUtilities->messageSendMail(null, $row["ASSUNTO"], self::tratarMensagemCorpoEmail($row), 'TECFOOD');
            $nomeRelatorio = self::geraNomeRelatorio($row);
            
            $attachments = [];
            $clearAttachments = true;
            
            foreach ($row['RELATORIOS'] as $urlRelatorio) {
                $attachment = new Attachment($urlRelatorio, $nomeRelatorio);
                $attachment->setType($tipo);
                $attachment->setIsStringAttachment(true);

                $attachments[] = $attachment;
            }
            
            if ($row['SENDCOPY'] === 'S') {
                $enderecosCopia[] = $this->oplUtilities->getCurrentEmail();
            }
            return $this->oplUtilities->enviaEmail('', $assunto, $enderecos, $corpoEmail, $enderecos, $enderecosCopia, $attachments, $clearAttachments);
            var_dump($this);
            die;
        } catch(\Exception $e) {
            return false;
        }
    }
    
    public function tratarMensagemCorpoEmail($row) 
    {
        $encontrar  = array("<i>[Nome Fornecedor]</i>", "<i>[link do portal]</i>", "<i>[nº do lote]</i>");
        $substituir = array($row['NMFANTFORN'], $row['DSLINKECOMMERCE'], $row['NRSEQAPURAF']);
        $message    = str_replace($encontrar, $substituir, $row['MENSAGEM']);
        $message   .= "<div style=\"font-size: 10px; margin-top: 20px; word-wrap: break-word;\"> Disponilizado em " . DateUtil::getDataAtual() . " </div>";
        
        return $message;
    }
    
    public function geraNomeRelatorio($row) 
    {
        $itensComposicaoNome = array(
            'CDFORNECED'    => $row['CDFORNECED'],
            'NMFANTFORN'    => $row['NMFANTFORN'],
            'SGESTADO'      => $row['SGESTADO'],
            'DTINIAPUPREC'  => self::formataDataForNomeRelatorio($row['DTINIAPUPREC']),
            'DTFINAPUPREC'  => self::formataDataForNomeRelatorio($row['DTFINAPUPREC']),
            'NRAPURPREC'    => $row['NRSEQAPURAF'],
            'DATA_ATUAL'    => DateUtil::getDataAtual()->format('Y-m-d-H-i-s')
        );
        
        return implode('_', $itensComposicaoNome) . '.' . strtolower($row['TYPE']);
    }
    
    public function formataDataForNomeRelatorio($date) 
    {
        return str_replace('/', '', explode(' ', $date)[0]);
    }
    
    private function setMessage($message) 
    {
        $this->message = $message;
    }
    
    private function getMessage() 
    {
        return $this->message;
    }
    
    private function setEmailEnviado($isEmailEnviado) 
    {
        $this->emailEnviado = $isEmailEnviado;
    }
    
    private function isEmailEnviado() 
    {
        return $this->emailEnviado;
    }
    
    private function setRelatorioGerado($isRelatorioGerado) 
    {
        $this->relatorioGerado = $isRelatorioGerado;
    }
    
    private function isRelatorioGerado() 
    {
        return $this->relatorioGerado;
    }
    
    private function setUrlRelatorio($urlRelatorio) 
    {
        $this->urlRelatorio = $urlRelatorio;
    }
    
    private function getUrlRelatorio() 
    {
        return $this->urlRelatorio;
    }
}